package com.jwl.driver.server.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.jwl.driver.server.constant.Constants;
import com.jwl.driver.server.dto.MemberQueryDto;
import com.jwl.driver.server.entity.TdMember;
import com.jwl.driver.server.mapper.TdMemberMapper;
import com.jwl.driver.server.service.ITdMemberService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jwl.driver.server.util.SecurityUtil;
import com.jwl.driver.server.util.TokenThreadUtil;
import com.jwl.driver.server.vo.MemberVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 会员类型表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
@Slf4j
public class TdMemberServiceImpl extends ServiceImpl<TdMemberMapper, TdMember> implements ITdMemberService {

    @Override
    public List<MemberVo> queryMember(MemberQueryDto queryDto) {

        List<MemberVo> resultList = new ArrayList<>();

        //获取用户已开通且未到期的会员（即使该会员已不售卖了）
        List<MemberVo> userMemberList = new ArrayList<>();
        try {
            if (StrUtil.isNotBlank(TokenThreadUtil.getToken())){
                userMemberList = queryUserMember(queryDto,SecurityUtil.getUserId());
            }
        }catch (Exception e){
            log.info("===================>用户没有登录");
        }

        Map<Integer, MemberVo> userMemberMap = userMemberList.stream().collect(Collectors.toMap(MemberVo::getMemberId, v -> v, (v1, v2) -> v1));

        //将用户购买的已停售但未过期的会员放在最前面
        List<MemberVo> expireMemberlist = userMemberList.stream().filter(s -> StrUtil.equals(s.getIsActive(), Constants.IS_ACTIVE_FALSE)).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(expireMemberlist)){
            resultList.addAll(expireMemberlist);
        }

        //获取现在可开通的会员
        List<MemberVo> memberList = this.getBaseMapper().queryMember(queryDto);
        if (CollectionUtil.isNotEmpty(memberList)){
            for (MemberVo memberVo : memberList) {
                if(userMemberMap.containsKey(memberVo.getMemberId())){
                    resultList.add(userMemberMap.get(memberVo.getMemberId()));
                }else {
                    resultList.add(memberVo);
                }
            }
        }

        return resultList;
    }

    @Override
    public List<MemberVo> queryUserMember(MemberQueryDto queryDto,Long userId) {
        return this.getBaseMapper().queryUserMember(queryDto,userId);
    }

}
