package com.jwl.driver.server.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.jwl.driver.server.constant.Constants;
import com.jwl.driver.server.dto.ProjectDto;
import com.jwl.driver.server.entity.TdProject;
import com.jwl.driver.server.entity.TdVideos;
import com.jwl.driver.server.mapper.TdProjectMapper;
import com.jwl.driver.server.service.ITdProjectService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jwl.driver.server.service.ITdVideosService;
import com.jwl.driver.server.vo.ProjectVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 考试项目表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
public class TdProjectServiceImpl extends ServiceImpl<TdProjectMapper, TdProject> implements ITdProjectService {

    @Autowired
    private ITdVideosService videosService;

    @Override
    public List<ProjectVo> queryProjectList(ProjectDto queryDto) {

        List<ProjectVo> resultList = new ArrayList<>();
        LambdaQueryWrapper<TdProject> queryWrapper = new LambdaQueryWrapper<TdProject>()
                .eq(Objects.nonNull(queryDto.getProjectId()),TdProject::getProjectId,queryDto.getProjectId())
                .eq(StrUtil.isNotBlank(queryDto.getType()),TdProject::getType,queryDto.getType())
                .eq(Objects.nonNull(queryDto.getCarTypeId()),TdProject::getCarTypeId,queryDto.getCarTypeId())
                .eq(StrUtil.isNotBlank(queryDto.getSubject()),TdProject::getSubject,queryDto.getSubject())
                .eq(StrUtil.isNotBlank(queryDto.getDriveType()),TdProject::getDriveType,queryDto.getDriveType())
                .eq(TdProject::getIsActive, Constants.IS_ACTIVE_TRUE)
                .orderByAsc(TdProject::getShowOrder);
        List<TdProject> tdProjects = this.getBaseMapper().selectList(queryWrapper);

        if (CollectionUtil.isNotEmpty(tdProjects)){
            for (TdProject tdProject : tdProjects) {
                ProjectVo projectVo = new ProjectVo();
                BeanUtils.copyProperties(tdProject,projectVo);
                //查询项目视频
                List<TdVideos> videoList = videosService.list(new QueryWrapper<TdVideos>().lambda().eq(TdVideos::getTypeId, tdProject.getProjectId()).orderByAsc(TdVideos::getShowOrder));
                if (CollectionUtil.isNotEmpty(videoList)){
                    projectVo.setVideoList(videoList);
                    projectVo.setImageUrl(videoList.get(0).getVideoImage());
                }
               resultList.add(projectVo);
            }
        }
        return resultList;
    }
}
