package com.jwl.driver.server.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.jwl.driver.server.entity.OrderPayInfo;
import com.jwl.driver.server.entity.TdMember;
import com.jwl.driver.server.entity.TdSysUserMember;
import com.jwl.driver.server.exception.BusinessException;
import com.jwl.driver.server.mapper.TdSysUserMemberMapper;
import com.jwl.driver.server.service.IOrderPayInfoService;
import com.jwl.driver.server.service.ITdMemberService;
import com.jwl.driver.server.service.ITdSysUserMemberService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jwl.driver.server.service.ITdSysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.Objects;

/**
 * <p>
 * 用户会员表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
public class TdSysUserMemberServiceImpl extends ServiceImpl<TdSysUserMemberMapper, TdSysUserMember> implements ITdSysUserMemberService {

    @Autowired
    private IOrderPayInfoService payInfoService;

    @Autowired
    private ITdMemberService memberService;

    @Autowired
    private ITdSysUserMemberService userMemberService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean purchaseMember(Long payId) {
        final LocalDateTime nowTime = LocalDateTime.now();

        OrderPayInfo payInfo = payInfoService.getById(payId);
        if (Objects.isNull(payInfo)){
            throw new BusinessException("订单信息不存在");
        }
        final Long userId = payInfo.getUserId();
        final Integer memberId = payInfo.getMemberId();
        // 查询会员 不管有效无效
        TdMember member = memberService.getById(memberId);
        if (Objects.isNull(member)){
            throw new BusinessException("购买的会员不存在");
        }
        //查看该用户是否存在有效会员
        LambdaQueryWrapper<TdSysUserMember> cond = new LambdaQueryWrapper<TdSysUserMember>()
                .eq(TdSysUserMember::getUserId,userId)
                .eq(TdSysUserMember::getMemberId,memberId)
                .gt(TdSysUserMember::getEndDate, nowTime);

        TdSysUserMember userMember = this.getOne(cond);
        //如果用户没购买该会员 或者该会员已过期就新建一个会员
        if (Objects.isNull(userMember)){
            userMember = new TdSysUserMember()
                    .setMemberId(memberId)
                    .setUserId(userId)
                    .setCreateTime(nowTime)
                    .setUpdateTime(nowTime)
                    .setStartDate(LocalDateTime.of(LocalDate.now(), LocalTime.MIN))
                    .setEndDate(LocalDateTime.of(LocalDate.now(), LocalTime.MIN).minusSeconds(1));
            boolean save = this.save(userMember);
            if (!save){
                throw new BusinessException("充值会员失败");
            }
        }
        LocalDateTime endTime = userMember.getEndDate();
        if (StrUtil.equals(member.getUnit(),"1")){
            endTime = endTime.plusDays(member.getDuration());
        }else if (StrUtil.equals(member.getUnit(),"2")){
            endTime = endTime.plusMonths(member.getDuration());
        } else if (StrUtil.equals(member.getUnit(),"3")) {
            endTime = endTime.plusYears(member.getDuration());
        }
        userMember.setEndDate(endTime);
        boolean update = this.update(userMember, cond);
        if (!update){
            throw new BusinessException("会员充值失败");
        }
        return Boolean.TRUE;
    }
}
